@@ -7,9 +7,6 @@ This extension provides commands that apply to all the nested repositories.
 It was inspired by the forest extension by Robin Farine.
 '''
 
-testedwith = '2.9 3.0 3.1 3.2 3.3 3.4 3.5'
-buglink = 'https://bitbucket.org/cedk/hgnested/issues'
-
 import os
 import time
 from functools import partial
@@ -17,126 +14,100 @@ from mercurial import hg
 from mercurial import commands, localrepo, hgweb
 from mercurial import util
 from mercurial import cmdutil
+from mercurial import extensions
 from mercurial import wireproto
 from mercurial.error import CapabilityError
 from mercurial.scmutil import walkrepos
 from mercurial.i18n import _
 
-try:
-    from mercurial import sshpeer, httppeer
-except ImportError:
-    from mercurial import sshrepo, httprepo
-    sshpeer, httppeer = None, None
+__version__ = '0.8'
+testedwith = '3.6 3.7 3.8 3.9'
+buglink = 'https://bitbucket.org/cedk/hgnested/issues'
 
-__version__ = '0.7'
+cmdtable = {}
+command = cmdutil.command(cmdtable)
 
 _nested_cache = {}
 _nested_refreshinterval = 20
 
-_capabilities_parent = wireproto.capabilities
 
-
-def capabilities(repo, proto):
-    caps = _capabilities_parent(repo, proto).split()
+def _capabilities(orig, repo, proto):
+    caps = orig(repo, proto)
     caps.append('nested')
-    return ' '.join(caps)
-wireproto.capabilities = capabilities
-wireproto.commands['capabilities'] = ((capabilities,) +
-        wireproto.commands['capabilities'][1:])
+    return caps
+extensions.wrapfunction(wireproto, '_capabilities', _capabilities)
 
 
+@wireproto.wireprotocommand('nested')
 def nested(repo, proto):
     '''Return a list of nested repositories.'''
     return "\n".join(repo.nested)
 
-wireproto.commands['nested'] = (nested, '')
+
+def reposetup(ui, repo):
+    if isinstance(repo, localrepo.localrepository):
+        class nestedrepo(repo.__class__):
+            @property
+            def nested(self):
+                '''Return a list of nested repositories.'''
+                cache = _nested_cache.setdefault(self.root, {
+                    'lastrefresh': 0,
+                    'nested': None,
+                    })
+                if (cache['lastrefresh'] + _nested_refreshinterval
+                        > time.time()):
+                    return cache['nested']
+                res = {}
+                paths = [self.root]
+                while paths:
+                    path = paths.pop()
+                    if os.path.realpath(path) in res:
+                        continue
+                    for root, dirs, files in os.walk(path):
+                        realroot = os.path.realpath(root)
+                        absroot = os.path.abspath(root)
+                        for dir in dirs[:]:
+                            if dir == '.hg':
+                                res[realroot] = absroot
+                                proot = os.path.join(root, '.hg', 'patches')
+                                if os.path.isdir(os.path.join(proot, '.hg')):
+                                    realproot = os.path.realpath(proot)
+                                    absproot = os.path.abspath(proot)
+                                    res[realproot] = absproot
+                                dirs.remove('.hg')  # Don't visit .hg directory
+                            else:
+                                path = os.path.join(root, dir)
+                                if (os.path.islink(path)
+                                        and os.path.realpath(path) not in res):
+                                    paths.append(path)
+                prefix = len(self.root) + 1
+                res = [x[prefix:] or '.' for x in res.itervalues()]
+                res.sort()
+                cache['nested'] = res
+                cache['lastrefresh'] = time.time()
+                return res
+        repo.__class__ = nestedrepo
+
+    if isinstance(repo, wireproto.wirepeer):
+        class nestedwirepeer(repo.__class__):
+            @property
+            def nested(self):
+                '''Return a list of nested repositories.'''
+                if not self.capable('nested'):
+                    raise util.Abort(_("Remote repository doesn't support "
+                        "the nested extension."))
+                return self._call('nested').splitlines()
+        repo.__class__ = nestedwirepeer
 
 
-@property
-def _localrepo_nested(self):
-    '''Return a list of nested repositories.'''
-    cache = _nested_cache.setdefault(self.root, {
-        'lastrefresh': 0,
-        'nested': None,
-        })
-    if cache['lastrefresh'] + _nested_refreshinterval > time.time():
-        return cache['nested']
-    res = {}
-    paths = [self.root]
-    while paths:
-        path = paths.pop()
-        if os.path.realpath(path) in res:
-            continue
-        for root, dirs, files in os.walk(path):
-            for dir in dirs[:]:
-                if dir == '.hg':
-                    res[os.path.realpath(root)] = os.path.abspath(root)
-                    proot = os.path.join(root, '.hg', 'patches')
-                    if os.path.isdir(os.path.join(proot, '.hg')):
-                        res[os.path.realpath(proot)] = os.path.abspath(proot)
-                    dirs.remove('.hg')  # Don't visit .hg directory
-                else:
-                    path = os.path.join(root, dir)
-                    if (os.path.islink(path)
-                            and os.path.realpath(path) not in res):
-                        paths.append(path)
-    prefix = len(self.root) + 1
-    res = [x[prefix:] or '.' for x in res.itervalues()]
-    res.sort()
-    cache['nested'] = res
-    cache['lastrefresh'] = time.time()
-    return res
-
-localrepo.localrepository.nested = _localrepo_nested
-
-
-@property
-def _sshpeer_nested(self):
-    '''Return a list of nested repositories.'''
-    if hasattr(self, 'capable'):
-        test = self.capable('nested')
-    else:
-        test = 'nested' in self.capabilities
-    if not test:
-        raise util.Abort(_("Remote repository doesn't support "
-            "the nested extension."))
-    if hasattr(self, '_call'):
-        return self._call('nested').splitlines()
-    return self.call('nested').splitlines()
-
-if sshpeer:
-    sshpeer.sshpeer.nested = _sshpeer_nested
-else:
-    sshrepo.sshrepository.nested = _sshpeer_nested
-
-
-@property
-def _httppeer_nested(self):
-    '''Return a list of nested repositories.'''
-    if hasattr(self, 'capable'):
-        test = self.capable('nested')
-    else:
-        test = 'nested' in self.capabilities
-    if not test:
-        raise util.Abort(_("Remote repository doesn't support "
-            "the nested extension."))
-    if hasattr(self, '_call'):
-        return self._call('nested').split()
-    return self.do_read('nested').split()
-
-if httppeer:
-    httppeer.httppeer.nested = _httppeer_nested
-else:
-    httprepo.httprepository.nested = _httppeer_nested
-
-
-_hgwebdir_refresh_parent = hgweb.hgwebdir_mod.hgwebdir.refresh
-
-
-def _hgwebdir_refresh(self):
-    if self.lastrefresh + self.refreshinterval > time.time():
+def _hgwebdir_refresh(orig, self):
+    refreshinterval = 20
+    if self.ui:
+        refreshinterval = self.ui.configint('web', 'refreshinterval',
+            refreshinterval)
+    if self.lastrefresh + refreshinterval > time.time():
         return
-    _hgwebdir_refresh_parent(self)
+    orig(self)
     for prefix, root in self.ui.configitems('collections'):
         prefix = util.pconvert(prefix)
         for path in walkrepos(root, followsym=True):
@@ -150,8 +121,8 @@ def _hgwebdir_refresh(self):
                 if repo not in self.repos:
                     self.repos.append(repo)
     self.lastrefresh = time.time()
-
-hgweb.hgwebdir_mod.hgwebdir.refresh = _hgwebdir_refresh
+extensions.wrapfunction(
+    hgweb.hgwebdir_mod.hgwebdir, 'refresh', _hgwebdir_refresh)
 
 
 def _nested_apply(ui, repo, function, status, *args, **kwargs):
@@ -181,6 +152,8 @@ def _nested_diff(ui, repo, *pats, **opts):
     cmdutil.diffordiffstat = diffordiffstat
 
 
+@command('^ndiff', cmdutil.findcmd('diff', commands.table)[1][1],
+    _('[OPTION]... ([-c REV] | [-r REV1 [-r REV2]]) [FILE]...'))
 def ndiff(ui, repo, *pats, **opts):
     '''diff nested repositories (or selected files)
 
@@ -190,6 +163,8 @@ def ndiff(ui, repo, *pats, **opts):
     _nested_apply(ui, repo, _nested_diff, False, *pats, **opts)
 
 
+@command('^nclone', cmdutil.findcmd('clone', commands.table)[1][1],
+    _('[OPTION]... SOURCE [DEST]'), norepo=True)
 def nclone(ui, source, dest=None, **opts):
     '''make a copy of an existing repository and all nested repositories
 
@@ -224,6 +199,9 @@ def nclone(ui, source, dest=None, **opts):
         ui.status('\n')
 
 
+@command('^nshare',
+    [('U', 'noupdate', None, _('do not create a working copy'))],
+    _('[-U] SOURCE [DEST]'), norepo=True)
 def nshare(ui, source, dest=None, noupdate=False):
     '''create a new shared repository and all nested repositories
     '''
@@ -247,6 +225,8 @@ def nshare(ui, source, dest=None, noupdate=False):
         ui.status('\n')
 
 
+@command('nincoming|nin', cmdutil.findcmd('incoming', commands.table)[1][1],
+    _('[-p] [-n] [-M] [-f] [-r REV]... [--bundle FILENAME] [SOURCE]'))
 def nincoming(ui, repo, dest=None, **opts):
     '''show changest not found in repository and all nested repositories
 
@@ -254,6 +234,8 @@ def nincoming(ui, repo, dest=None, **opts):
     _nested_apply(ui, repo, commands.incoming, True, dest=dest, **opts)
 
 
+@command('noutgoing|nout', cmdutil.findcmd('outgoing', commands.table)[1][1],
+    _('[-M] [-p] [-n] [-f] [-r REV]... [DEST]'))
 def noutgoing(ui, repo, dest=None, **opts):
     '''show changesest not found in the destination and all nested repositories
 
@@ -261,6 +243,8 @@ def noutgoing(ui, repo, dest=None, **opts):
     _nested_apply(ui, repo, commands.outgoing, True, dest=dest, **opts)
 
 
+@command('npull', cmdutil.findcmd('pull', commands.table)[1][1],
+    _('[-u] [-f] [-r REV]... [-e CMD] [--remotecmd CMD] [SOURCE]'))
 def npull(ui, repo, source="default", **opts):
     '''pull changes from the specified source and all nested repositories
 
@@ -287,6 +271,8 @@ def npull(ui, repo, source="default", **opts):
     _nested_apply(ui, repo, commands.pull, True, source=source, **opts)
 
 
+@command('^npush', cmdutil.findcmd('push', commands.table)[1][1],
+    _('[-f] [-r REV]... [-e CMD] [--remotecmd CMD] [DEST]'))
 def npush(ui, repo, dest=None, **opts):
     '''push changes to the specified destination and all nested repositories
 
@@ -311,6 +297,8 @@ def npush(ui, repo, dest=None, **opts):
     _nested_apply(ui, repo, commands.push, True, dest=dest, **opts)
 
 
+@command('^nstatus|nst', cmdutil.findcmd('status', commands.table)[1][1],
+    _('[OPTION]... [FILE]...'))
 def nstatus(ui, repo, *pats, **opts):
     '''show changed files in the working directory and all nested repositories
 
@@ -320,6 +308,8 @@ def nstatus(ui, repo, *pats, **opts):
     _nested_apply(ui, repo, commands.status, True, *pats, **opts)
 
 
+@command('^nupdate|nup', cmdutil.findcmd('update', commands.table)[1][1],
+    _('[-c] [-C] [-d DATE] [[-r] REV]'))
 def nupdate(ui, repo, node=None, rev=None, clean=False, date=None, check=False,
         **opts):
     '''update working directory and all nested repositories
@@ -345,34 +335,3 @@ def nupdate(ui, repo, node=None, rev=None, clean=False, date=None, check=False,
             check=check)
     _nested_apply(ui, repo, commands.update, True, node=node, rev=rev,
             clean=clean, date=date, check=check)
-
-cmdtable = {
-    '^ndiff': (ndiff,
-        cmdutil.findcmd('diff', commands.table)[1][1],
-        _('[OPTION]... ([-c REV] | [-r REV1 [-r REV2]]) [FILE]...')),
-    '^nclone': (nclone,
-        cmdutil.findcmd('clone', commands.table)[1][1],
-        _('[OPTION]... SOURCE [DEST]')),
-    '^nshare': (nshare,
-        [('U', 'noupdate', None, _('do not create a working copy'))],
-        ('[-U] SOURCE [DEST]')),
-    'nincoming|nin': (nincoming,
-        cmdutil.findcmd('incoming', commands.table)[1][1],
-        _('[-p] [-n] [-M] [-f] [-r REV]... [--bundle FILENAME] [SOURCE]')),
-    'noutgoing|nout': (noutgoing,
-        cmdutil.findcmd('outgoing', commands.table)[1][1],
-        _('[-M] [-p] [-n] [-f] [-r REV]... [DEST]')),
-    'npull': (npull,
-        cmdutil.findcmd('pull', commands.table)[1][1],
-        _('[-u] [-f] [-r REV]... [-e CMD] [--remotecmd CMD] [SOURCE]')),
-    '^npush': (npush,
-        cmdutil.findcmd('push', commands.table)[1][1],
-        _('[-f] [-r REV]... [-e CMD] [--remotecmd CMD] [DEST]')),
-    '^nstatus|nst': (nstatus,
-        cmdutil.findcmd('status', commands.table)[1][1],
-        _('[OPTION]... [FILE]...')),
-    '^nupdate|nup': (nupdate,
-        cmdutil.findcmd('update', commands.table)[1][1],
-        _('[-c] [-C] [-d DATE] [[-r] REV]')),
-}
-commands.norepo += " nclone nshare"
